﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.ComponentModel.Composition;
using VVVV.Core;
using VVVV.PluginInterfaces.V2;
using VVVV.Hosting;
using VVVV.Hosting.Interfaces;

namespace VVVV.Nodes
{
    public class CustomObject : IPartImportsSatisfiedNotification
    {
        [Import]
        public IHDEHost FHDEHost;

        // simple fields which supposed to increment on every frame at different stages
        public int PrepareGraphIncrement = 0;
        public int PresentIncrement = 0;

        public CustomObject() { }

        // debug doesn't even stop here
        public void OnImportsSatisfied()
        {
            FHDEHost.MainLoop.OnPrepareGraph += OnPrepareGraph;
            FHDEHost.MainLoop.OnPresent += OnPresent;
        }

        private void OnPresent(object sender, EventArgs e)
        {
            this.PresentIncrement++;
        }

        private void OnPrepareGraph(object sender, EventArgs e)
        {
            this.PrepareGraphIncrement++;
        }
    }
    
    [PluginInfo(Name = "Source",
                Category = "MainloopEventsExample"
                )]
    public class MainloopEventsExampleSourceNode : IPluginEvaluate
    {
        // It will create a new object and replace the last one
        [Input("Construct New", IsBang=true)]
        public ISpread<bool> FConstruct;

        // It will remove the object from this node only
        [Input("Remove from here", IsBang = true)]
        public ISpread<bool> FRemove;

        [Output("Output")]
        public ISpread<CustomObject> FOut;

        public bool Valid = false;

        public void Evaluate(int SpreadMax)
        {
            if (FConstruct[0]) Valid = true;
            if (FRemove[0]) Valid = false;
            if (Valid)
            {
                FOut.SliceCount = 1;
                if (FConstruct[0]) FOut[0] = new CustomObject();
            }
            else FOut.SliceCount = 0;
        }
    }

    [PluginInfo(Name = "Sink",
                Category = "MainloopEventsExample"
                )]
    public class MainloopEventsExampleSinkNode : IPluginEvaluate
    {
        [Input("Input", IsBang = true)]
        public ISpread<CustomObject> FInput;

        // It will hold the object at this node only
        [Input("Hold it here", IsBang = true)]
        public ISpread<bool> FHold;

        [Output("Output")]
        public ISpread<CustomObject> FOut;

        public void Evaluate(int SpreadMax)
        {
            if (FInput.SliceCount > 0)
            {
                if (FInput[0] != null)
                {
                    FOut.SliceCount = 1;
                    if (FHold[0]) FOut[0] = FInput[0];
                }
                else FOut.SliceCount = 0;
            }
            else FOut.SliceCount = 0;
        }
    }

    [PluginInfo(Name = "Split",
                Category = "MainloopEventsExample"
                )]
    public class MainloopEventsExampleSplitNode : IPluginEvaluate
    {
        [Input("Input", IsBang = true)]
        public ISpread<CustomObject> FInput;

        [Output("PrepareGraph")]
        public ISpread<int> FPrepareGraph;

        [Output("Present")]
        public ISpread<int> FPresent;

        public void Evaluate(int SpreadMax)
        {
            if (FInput.SliceCount > 0)
            {
                if (FInput[0] != null)
                {
                    FPrepareGraph.SliceCount = 1;
                    FPresent.SliceCount = 1;
                    FPrepareGraph[0] = FInput[0].PrepareGraphIncrement;
                    FPresent[0] = FInput[0].PresentIncrement;
                }
                else
                {
                    FPrepareGraph.SliceCount = 0;
                    FPresent.SliceCount = 0;
                }
            }
            else
            {
                FPrepareGraph.SliceCount = 0;
                FPresent.SliceCount = 0;
            }
        }
    }
}
